@@ -138,6 +138,7 @@ TermDef       StyleTerms[] = {
 	{TF_NO_MYNAME_PREPENDING, "UseAnyViewport", 14, TT_FLAG, DATABASE_UseAnyViewport_ID, NULL},
 	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,Fullscreen,TT_FLAG,NULL),
 	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,NoFullscreen,TT_FLAG,NULL),
+	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,WindowOpacity,TT_INTEGER,NULL),
 	{0, NULL, 0, 0, 0}
 };
 
@@ -239,6 +240,7 @@ style_init (name_list * nl)
 	nl->ViewportX = -1;
 	nl->ViewportY = -1;
 	nl->gravity = NorthWestGravity;
+	nl->window_opacity = 100 ;
 
 	nl->off_buttons = 0;
 	nl->on_buttons = 0;
@@ -300,6 +302,8 @@ style_copy (name_list * to, name_list * from)
 			to->layer = from->layer;
 		if (get_flags (from->set_data_flags, STYLE_GRAVITY))
 			to->gravity = from->gravity;
+		if (get_flags (from->set_data_flags, STYLE_WINDOW_OPACITY))
+			to->window_opacity = from->window_opacity;
 
 		to->set_flags |= from->set_flags;
 		to->flags |= (from->set_flags & from->flags);
@@ -435,6 +439,9 @@ ParseSingleStyle (FreeStorageElem * storage, name_list * style)
 			 set_string_value (&(style->icon_file), item.data.string, &(style->set_data_flags), STYLE_ICON);
 			 if (strlen (style->icon_file) <= 0)
 				 clear_flags (style->set_data_flags, STYLE_ICON);
+             /* regardles of weather the file was valid - indicate that app should have an icon !!! */
+			 set_flags (style->set_flags, STYLE_ICON);
+			 set_flags (style->flags, STYLE_ICON);
 			 break;
 		 case DATABASE_FocusStyle_ID:
 		 case DATABASE_UnfocusStyle_ID:
@@ -499,6 +506,12 @@ ParseSingleStyle (FreeStorageElem * storage, name_list * style)
 		 case DATABASE_ViewportY_ID:
 			 set_flags (style->set_data_flags, STYLE_VIEWPORTY);
 			 style->ViewportY = item.data.integer;
+			 LOCAL_DEBUG_OUT( "style->ViewportY = %d", style->ViewportY );
+			 break;
+		 case DATABASE_WindowOpacity_ID:
+			 set_flags (style->set_data_flags, STYLE_WINDOW_OPACITY);
+			 style->window_opacity = item.data.integer;
+			 LOCAL_DEBUG_OUT( "style->window_opacity = %d", style->window_opacity );
 			 break;
 		 case DATABASE_Frame_ID:
 			 set_string_value (&(style->frame_name), item.data.string, &(style->set_data_flags), STYLE_FRAME);
@@ -568,6 +581,32 @@ ParseDatabaseOptions (const char *filename, char *myname)
 	return config;
 }
 
+name_list    *
+string2DatabaseStyle (char *style_txt)
+{
+	ConfigData    cd ;
+	ConfigDef    *ConfigReader;
+	name_list    *style = NULL;
+	FreeStorageElem *Storage = NULL;
+
+	cd.data = style_txt ;
+	ConfigReader = InitConfigReader ("afterstep", &StyleSyntax, CDT_Data, cd, NULL);
+	if (!ConfigReader)
+		return NULL;
+	
+	PrintConfigReader (ConfigReader);
+	
+	ParseConfig (ConfigReader, &Storage);
+
+	if ((style = style_new (NULL)) != NULL )
+		ParseSingleStyle (Storage, style);
+
+	DestroyFreeStorage (&Storage);
+	DestroyConfig (ConfigReader);
+	return style;
+}
+
+
 Bool ReloadASDatabase()
 {
 	char *configfile  = make_session_file(Session, DATABASE_FILE, False ); 
