@@ -43,6 +43,7 @@
 #include "../../libAfterStep/aswindata.h"
 #include "../../libAfterStep/balloon.h"
 #include "../../libAfterStep/event.h"
+#include "../../libAfterStep/shape.h"
 
 #include "../../libAfterConf/afterconf.h"
 
@@ -59,7 +60,12 @@ typedef struct ASWinListColumn
     unsigned int width, height ;
 }ASWinListColumn;
 
+#define ASWL_Client_NoCollides	(0x01<<0)
+
 typedef struct {
+
+#define ASWL_Mapped		(0x01<<0)
+	ASFlagType flags ;
     int windows_num ;
 
     ASWindowData       *focused;
@@ -76,6 +82,7 @@ typedef struct {
     unsigned short col_width[MAX_WINLIST_WINDOW_COUNT];
     unsigned short col_x[MAX_WINLIST_WINDOW_COUNT];
 
+    unsigned int min_col_width ;
     unsigned int max_item_height ;
     unsigned int columns_num, rows_num;
 
@@ -86,9 +93,19 @@ typedef struct {
 
 	ASTBarProps *tbar_props ;
 
+    ASWindowData       *self;
+
+	ASDatabaseRecord db_rec ; 	
+	int border_width ; 
+
+	/* this will be used instead of what we get in frame_rect from AS, 
+	   as those messages could arrive too late :*/
+	int frame_x, frame_y ;
+	unsigned int frame_width, frame_height, frame_bw ; 
+
 }ASWinListState ;
 
-ASWinListState WinListState = { 0, NULL, None, NULL, NULL };
+ASWinListState WinListState;
 
 
 #define WINLIST_BACK_URGENT		BACK_DEFAULT
@@ -103,6 +120,24 @@ ASWinListState WinListState = { 0, NULL, None, NULL, NULL };
  */
 
 WinListConfig *Config = NULL ;
+int MaxRows_override = 0 ;
+int MaxColumns_override = 0 ;
+
+CommandLineOpts WinList_cmdl_options[3] =
+{
+	{NULL, "max-rows","Overrides WinList MaxRows setting", NULL, handler_set_int, &MaxRows_override, 0, CMO_HasArgs },
+	{NULL, "max-cols","Overrides WinList MaxColumns setting", NULL, handler_set_int, &MaxColumns_override, 0, CMO_HasArgs },
+    {NULL, NULL, NULL, NULL, NULL, NULL, 0 }
+};
+
+void
+WinList_usage (void)
+{
+	printf (OPTION_USAGE_FORMAT " [--rows rows] [--cols cols] n m\n", MyName );
+	print_command_line_opt("standard_options are ", as_standard_cmdl_options, ASS_Restarting);
+	print_command_line_opt("additional options are ", WinList_cmdl_options, 0);
+	exit (0);
+}
 
 
 /**********************************************************************/
@@ -116,7 +151,7 @@ void process_message (send_data_type type, send_data_type *body);
 void DispatchEvent (ASEvent * Event);
 Window make_winlist_window();
 void add_winlist_button( ASTBarData *tbar, ASWindowData *wd );
-void refresh_winlist_button( ASTBarData *tbar, ASWindowData *wd, Bool focus_only );
+Bool refresh_winlist_button( ASTBarData *tbar, ASWindowData *wd, Bool focus_only );
 void delete_winlist_button( ASTBarData *tbar, ASWindowData *wd );
 Bool rearrange_winlist_window( Bool dont_resize_main_canvas );
 unsigned int find_button_by_position( int x, int y );
@@ -124,19 +159,39 @@ void press_winlist_button( ASWindowData *wd );
 void release_winlist_button( ASWindowData *wd, int button );
 void update_winlist_styles();
 void DeadPipe(int);
+void switch_deskviewport( int new_desk, int new_vx, int new_vy );
+Bool check_avoid_collision();
 
 int
 main( int argc, char **argv )
 {
+	int i ;
+
+	memset( &WinListState, 0x00, sizeof(WinListState));
+	
     /* Save our program name - for error messages */
 	set_DeadPipe_handler(DeadPipe);
-    InitMyApp (CLASS_WINLIST, argc, argv, NULL, NULL, 0 );
+    InitMyApp (CLASS_WINLIST, argc, argv, WinList_usage, NULL, 0 );
 	LinkAfterStepConfig();
 
+    for( i = 1 ; i< argc ; ++i)
+	{
+		LOCAL_DEBUG_OUT( "argv[%d] = \"%s\", original argv[%d] = \"%s\"", i, argv[i], i, MyArgs.saved_argv[i]);	  
+		if( argv[i] != NULL )
+		{ 	
+	    	if( strcmp( argv[i] , "--max-rows" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
+				MaxRows_override = atoi( argv[++i] );
+	    	else if( strcmp( argv[i] , "--max-cols" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
+				MaxColumns_override = atoi( argv[++i] );
+		}
+	}
+
+LOCAL_DEBUG_OUT( "Max overrides %d, %d", MaxColumns_override, MaxRows_override );
     set_signal_handler( SIGSEGV );
 
     ConnectX( ASDefaultScr, EnterWindowMask );
     if( ConnectAfterStep (M_FOCUS_CHANGE |
+                      M_NEW_DESKVIEWPORT |
                       M_DESTROY_WINDOW |
                       WINDOW_CONFIG_MASK |
                       WINDOW_NAME_MASK |
@@ -154,18 +209,22 @@ main( int argc, char **argv )
 	Config = AS_WINLIST_CONFIG(parse_asmodule_config_all( WinListConfigClass )/*asm_config*/);
 	
 /* 	LoadConfig ("winlist", GetOptions); */
-	CheckWinListConfigSanity(Config, &(MyArgs.geometry), MyArgs.gravity);
+	CheckWinListConfigSanity(Config, &(MyArgs.geometry), MyArgs.gravity, MaxColumns_override, MaxRows_override);
+LOCAL_DEBUG_OUT( "Max cols/rows %d, %d", Config->MaxColumns, Config->MaxRows );
+	
 	ReloadASDatabase();
 	ReloadCategories(True);
     
 	retrieve_winlist_astbar_props();
 		
+LOCAL_DEBUG_OUT( "Max cols/rows %d, %d", Config->MaxColumns, Config->MaxRows );
 	SetWinListLook();
 
+	clear_flags(WinListState.flags, ASWL_Mapped ); 
     WinListState.main_window = make_winlist_window();
     WinListState.main_canvas = create_ascanvas( WinListState.main_window );
     set_root_clip_area( WinListState.main_canvas );
-    rearrange_winlist_window( False );
+//    rearrange_winlist_window( False );
 
 	WinListState.postpone_display = True ;
 
@@ -307,6 +366,32 @@ GetOptions (const char *filename)
 #endif	
 }
 
+Bool 
+match_NoCollides( char *name )
+{
+	int i ; 
+	Bool no_collides = False ;
+	for( i = 0 ; i < Config->NoCollides_nitems ; ++i ) 
+		if( Config->NoCollides_wrexp ) 
+			if( match_wild_reg_exp( name, Config->NoCollides_wrexp[i] ) == 0 )
+			{
+				no_collides = True ; 
+				break;
+			}	
+				
+	if( no_collides ) 
+	{
+		for( i = 0 ; i < Config->AllowCollides_nitems ; ++i ) 
+			if( Config->AllowCollides_wrexp ) 
+				if( match_wild_reg_exp( name, Config->AllowCollides_wrexp[i] ) == 0 )
+				{
+					no_collides = False ; 
+					break;
+				}	
+	}
+		
+	return no_collides ; 
+}
 
 /****************************************************************************/
 /* PROCESSING OF AFTERSTEP MESSAGES :                                       */
@@ -317,10 +402,21 @@ process_message (send_data_type type, send_data_type *body)
     LOCAL_DEBUG_OUT( "received message %lX", type );
 	WinListState.last_message_time = time(NULL) ;
 
-	if( type == M_END_WINDOWLIST )
+	if( type == M_NEW_DESKVIEWPORT )
+	{
+    	LOCAL_DEBUG_OUT("M_NEW_DESKVIEWPORT(desk = %ld,Vx=%ld,Vy=%ld)", body[2], body[0], body[1]);
+        switch_deskviewport( body[2], body[0], body[1] );
+	}else if( type == M_END_WINDOWLIST )
 	{
 		WinListState.postpone_display = False ;
 		rearrange_winlist_window( False );
+		if( !get_flags( WinListState.flags, ASWL_Mapped ) )	
+		{
+			XMapRaised (dpy, WinListState.main_window);
+			set_flags( WinListState.flags, ASWL_Mapped );
+			ASSync(False);
+			sleep_a_millisec(100);
+		}
 	}else if( (type&WINDOW_PACKET_MASK) != 0 )
 	{
 		struct ASWindowData *wd = fetch_window_by_id( body[0] );
@@ -332,17 +428,55 @@ process_message (send_data_type type, send_data_type *body)
 		res = handle_window_packet( type, body, &wd );
 		if( res == WP_DataCreated )
         {
-            if( WinListState.windows_num < MAX_WINLIST_WINDOW_COUNT &&
-                WinListState.windows_num < Config->MaxRows*Config->MaxColumns )
-            {
-                if( !get_flags( Config->flags, ASWL_UseSkipList ) ||
-                    !get_flags( wd->flags, AS_SkipWinList ) )
-                    add_winlist_button( tbar, wd );
-            }
+			if( wd->client == WinListState.main_window ) 
+			{
+				WinListState.self = wd ;
+			    XSelectInput (dpy, wd->frame, StructureNotifyMask );
+            }else 
+			{
+				if( WinListState.windows_num < MAX_WINLIST_WINDOW_COUNT &&
+                	WinListState.windows_num < Config->MaxRows*Config->MaxColumns )
+	            {
+    	            if( !get_flags( Config->flags, ASWL_UseSkipList ) ||
+        	            !get_flags( wd->flags, AS_SkipWinList ) )
+            	        add_winlist_button( tbar, wd );
+	            }
+			}
         }else if( res == WP_DataChanged )
-			refresh_winlist_button( tbar, wd, (type == M_FOCUS_CHANGE) );
-		else if( res == WP_DataDeleted )
-            delete_winlist_button( tbar, saved_wd );
+		{
+			if( (type & WINDOW_NAME_MASK) && type != M_WINDOW_NAME_MATCHED )
+			{
+		        CARD32 *pbuf = &(body[4]);
+		        char *new_name = deserialize_string( &pbuf, NULL );
+
+				if( new_name && match_NoCollides( new_name ) ) 
+				{
+					set_flags( wd->module_flags, ASWL_Client_NoCollides );
+				}else if( get_flags( wd->module_flags, ASWL_Client_NoCollides ) )
+				{
+					if( !match_NoCollides( wd->window_name ) && 
+						!match_NoCollides( wd->icon_name ) && 
+						!match_NoCollides( wd->res_name ) && 
+						!match_NoCollides( wd->res_class ) )
+					{
+						clear_flags( wd->module_flags, ASWL_Client_NoCollides );
+					}
+				}
+				if( new_name ) free(new_name );
+			}
+			if( !refresh_winlist_button( tbar, wd, (type == M_FOCUS_CHANGE) ) )
+			{
+				if(wd != WinListState.self && get_flags( wd->module_flags, ASWL_Client_NoCollides ) )
+					if( !WinListState.postpone_display && check_avoid_collision() )
+						rearrange_winlist_window( False );
+			}
+		}else if( res == WP_DataDeleted )
+		{
+			if( WinListState.self == saved_wd ) /* just in case */
+				WinListState.self = NULL ;
+            else 
+				delete_winlist_button( tbar, saved_wd );
+		}
 
 	}
 }
@@ -370,14 +504,28 @@ DispatchEvent (ASEvent * event)
     switch (event->x.type)
     {
 	    case ConfigureNotify:
+			if( WinListState.self && event->x.xconfigure.window == WinListState.self->frame ) 
+			{
+				WinListState.frame_x = event->x.xconfigure.x ; 
+				WinListState.frame_y = event->x.xconfigure.y ; 
+				WinListState.frame_width = event->x.xconfigure.width ; 
+				WinListState.frame_height = event->x.xconfigure.height ; 
+				WinListState.frame_bw = event->x.xconfigure.border_width ; 
+				if( Config->gravity == NorthEastGravity || Config->gravity == SouthEastGravity )
+					Config->anchor_x = WinListState.frame_x + WinListState.frame_width+WinListState.frame_bw*2 ; 
+				if( Config->gravity == SouthWestGravity || Config->gravity == SouthEastGravity )
+					Config->anchor_y = WinListState.frame_y + WinListState.frame_height+WinListState.frame_bw*2 ; 
+			}else
             {
                 ASFlagType changes = handle_canvas_config( WinListState.main_canvas );
                 if( changes != 0 )
                     set_root_clip_area( WinListState.main_canvas );
 
                 if( get_flags( changes, CANVAS_RESIZED ) )
-                    rearrange_winlist_window(True);
-                else if( changes != 0 )        /* moved - update transparency ! */
+				{
+					if( !WinListState.postpone_display )
+	                    rearrange_winlist_window(True);
+				}else if( changes != 0 )        /* moved - update transparency ! */
                 {
                     int i ;
                     for( i = 0 ; i < WinListState.windows_num ; ++i )
@@ -443,7 +591,8 @@ DispatchEvent (ASEvent * event)
 					SetWinListLook();
 					/* now we need to update everything */
 					update_winlist_styles();
-					rearrange_winlist_window( False );
+					if( !WinListState.postpone_display )
+						rearrange_winlist_window( False );
                 	for( i = 0 ; i < WinListState.windows_num ; ++i )
 						refresh_winlist_button( WinListState.window_order[i]->bar, WinListState.window_order[i], False );
 				}else if( event->x.xproperty.atom == _AS_TBAR_PROPS )
@@ -479,9 +628,10 @@ make_winlist_window()
 	Window        w;
 	XSizeHints    shints;
 	ExtendedWMHints extwm_hints ;
-	int x, y ;
+	int x =0, y = 0;
     unsigned int width = max(Config->MinSize.width,1);
     unsigned int height = max(Config->MinSize.height,1);
+	static char *WinList_names[3] = {NULL, CLASS_WINLIST, NULL };
 
 	switch( Config->gravity )
 	{
@@ -504,16 +654,27 @@ make_winlist_window()
 			break;
 	}
 
-	w = create_visual_window( Scr.asv, Scr.Root, x, y, width, height, 0, InputOutput, 0, NULL);
-    set_client_names( w, "Window List", "Window List", CLASS_WINLIST, MyName );
+	w = create_visual_window( Scr.asv, Scr.Root, x, y, 1, 1, 0, InputOutput, 0, NULL);
+    set_client_names( w, MyName, MyName, AS_MODULE_CLASS, CLASS_WINLIST );
+
+	WinList_names[0] = MyName ;
+
+#define APPEND_DBSTYLE_TEXT(text) 	\
+			do { if( !first ) strcat( buf, ", "); else first = False ; strcat( buf, text); } while(0)
+
+	if ( fill_asdb_record (Database, &(WinList_names[0]), &WinListState.db_rec, False) != NULL )
+	{
+		if (get_flags (WinListState.db_rec.set_data_flags, STYLE_BORDER_WIDTH))
+			WinListState.border_width = WinListState.db_rec.border_width;
+	}
 
 	shints.flags = USPosition|USSize|PMinSize|PMaxSize|PBaseSize|PWinGravity;
 	shints.min_width = shints.min_height = 4;
 	shints.max_width = (Config->MaxSize.width>0)?Config->MaxSize.width:Scr.MyDisplayWidth;
 	shints.max_height = (Config->MaxSize.height>0)?Config->MaxSize.height:Scr.MyDisplayHeight;
 	shints.base_width = shints.base_height = 4;
-	shints.win_gravity = Config->gravity ;
-
+	shints.win_gravity = Config->gravity ; /* should be StaticGravity for some position manipulations */
+LOCAL_DEBUG_OUT( "gravity = %d", Config->gravity );
 	extwm_hints.pid = getpid();
 	extwm_hints.flags = EXTWM_PID|EXTWM_TypeSet|EXTWM_StateSet ;
 	extwm_hints.state_flags = EXTWM_StateSkipTaskbar|EXTWM_StateSkipPager ;
@@ -522,8 +683,6 @@ make_winlist_window()
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
 	set_client_cmd (w);
 
-	/* showing window to let user see that we are doing something */
-	XMapRaised (dpy, w);
 	/* final cleanup */
 	XFlush (dpy);
 	sleep (1);								   /* we have to give AS a chance to spot us */
@@ -554,6 +713,285 @@ render_winlist_button( ASTBarData *tbar )
 	return False ;
 }
 
+/* collision avoidance code  (*WinListNoCollides): */
+/*************************************************************************/
+/* here we build vector of rectangles, representing one available
+ * space each :
+ */
+/*************************************************************************/
+Bool
+winlist_get_free_rectangles_iter_func(void *data, void *aux_data)
+{
+    ASVector *list = (ASVector*)aux_data ;
+    ASWindowData *wd = (ASWindowData*)data ;
+    int min_vx = 0, max_vx = Scr.MyDisplayWidth ;
+    int min_vy = 0, max_vy = Scr.MyDisplayHeight ;
+	
+    if( wd && wd->client != WinListState.main_window && 
+		!get_flags(wd->state_flags, AS_Shaded ) && 
+		get_flags( wd->module_flags, ASWL_Client_NoCollides ) && 
+		(wd->desk == Scr.CurrentDesk || get_flags(wd->state_flags, AS_Sticky)))
+    {
+		ASRectangle *r = &(wd->frame_rect);
+        int x, y;
+
+        if( get_flags(wd->state_flags, AS_Fullscreen ) )
+			return True;
+		
+        if( get_flags(wd->state_flags, AS_Iconic ) )
+			r = &(wd->icon_rect);
+			
+		x = r->x ; 
+		y = r->y ; 
+
+		if( get_flags(wd->state_flags, AS_Sticky|AS_Iconic ) == 0)
+		{
+			x -= Scr.Vx ; 
+			y -= Scr.Vy ; 
+		}
+		LOCAL_DEBUG_OUT( "subtracting rectangle of \"%s\" %lux%lu%+d%+d", wd->window_name, r->width, r->height, x, y );
+        if( x+(int)r->width >= min_vx && x < max_vx && y+(int)r->height >= min_vy && y < max_vy )
+            subtract_rectangle_from_list( list, x, y, x+(int)r->width, y+(int)r->height );
+    }
+
+    return True;
+}
+
+static ASVector *
+winlist_build_free_space_list()
+{
+    ASVector *list = create_asvector( sizeof(XRectangle) );
+    XRectangle seed_rect ;
+
+    /* must seed the list with the single rectangle representing the area : */
+    seed_rect.x = 0 ;
+    seed_rect.y = 0 ;
+    seed_rect.width = Scr.MyDisplayWidth;
+    seed_rect.height = Scr.MyDisplayHeight ;
+
+    append_vector( list, &seed_rect, 1 );
+
+	iterate_window_data( winlist_get_free_rectangles_iter_func, (void*)list );
+
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
+    print_rectangles_list( list );
+#endif
+
+    return list;
+}
+
+#define FIT_TO_RECT(p,s,rp,rs) \
+			do{ if( (p) < (int)(rp) ) (p) = (rp) ; \
+			if( (int)(p)+(int)(s)  > (int)(rp) + (int)(rs) ) { \
+				p = (int)((rp)+(rs)) - (s) ; \
+				if( p < (int)(rp) ) { s -= (int)(rp) - (p) ; p = rp ;} \
+			}}while(0)
+
+
+void
+winlist_avoid_collision( int *px, int *py, unsigned int *pmax_width, unsigned int *pmax_height, int min_width, int min_height )
+{
+	int x = *px ;
+	int y = *py ;
+	int w = *pmax_width ; 
+	int h = *pmax_height ; 	
+	ASVector *free_space_list = winlist_build_free_space_list(); 
+    XRectangle *rects = PVECTOR_HEAD(XRectangle,free_space_list);
+	int i = PVECTOR_USED(free_space_list); 
+	int selected = -1 ;
+	int selected_factor = 0;
+	int frame_add_v = 0, frame_add_h = 0 ;
+	
+	if( WinListState.self) 
+	{
+		frame_add_h = (int)WinListState.frame_width - (int)WinListState.frame_width + (int)WinListState.frame_bw*2 ; 
+		frame_add_v = (int)WinListState.frame_height - (int)WinListState.frame_height + (int)WinListState.frame_bw*2 ; 
+	}else
+	{
+		frame_add_h = frame_add_v = WinListState.border_width*2 ; 
+	}
+	
+	LOCAL_DEBUG_OUT( "requested geometry = %dx%d%+d%+d, min_size = %dx%d, frame_borders = %+d%+d", w, h, x, y, min_width, min_height, frame_add_h, frame_add_v );
+    if( get_flags( Config->flags, ASWL_RowsFirst ) )
+	{	/* strategy # 1 - find closest possible rectangle without changing y position */
+		int max_y = y+min_height ;
+		int min_y = y ; 
+		if( Config->gravity == SouthWestGravity || Config->gravity == SouthEastGravity )
+		{
+			max_y = Config->anchor_y ; 
+			min_y = max_y - min_height ; 
+			y = Config->anchor_y - h ; 
+		}
+    	while( --i >= 0 )
+			if( rects[i].y <= min_y && max_y <= rects[i].y + rects[i].height )
+				if( rects[i].width > selected_factor ) 
+				{
+					selected = i ;
+					selected_factor = rects[i].width ;
+				}
+		if( selected >= 0 ) 
+		{
+			if( rects[selected].x+rects[selected].width < Scr.MyDisplayWidth-1 ) 
+				rects[selected].width -= Config->NoCollidesSpacing ;
+			rects[selected].width -= frame_add_h ;
+			if( rects[selected].x > 0 ) 
+			{
+				rects[selected].width -= Config->NoCollidesSpacing; 
+				rects[selected].x += Config->NoCollidesSpacing ;
+			}
+			FIT_TO_RECT(x,w,rects[selected].x,rects[selected].width);
+			if( y < rects[selected].y )
+				y = rects[selected].y ;
+		}
+	}else
+	{
+		int max_x = min_width + x ;
+		int min_x = x ;
+		
+		if( Config->gravity == NorthEastGravity || Config->gravity == SouthEastGravity )
+		{
+			max_x = Config->anchor_x ; 
+			min_x = max_x - min_width ; 
+			x = Config->anchor_x - w ; 
+		}
+		/* strategy # 2 - find closest possible rectangle without changing x position */
+    	while( --i >= 0 )
+			if( rects[i].x <= x && max_x <= rects[i].x + rects[i].width )
+				if( rects[i].height > selected_factor ) 
+				{
+					selected = i ;
+					selected_factor = rects[i].height ;
+				}
+		if( selected >= 0 ) 
+		{
+			
+			if( rects[selected].y+rects[selected].height < Scr.MyDisplayHeight-1 ) 
+				rects[selected].height -=  Config->NoCollidesSpacing; 
+			rects[selected].height -= frame_add_v; 
+			if( rects[selected].y > 0 ) 
+			{
+				rects[selected].height -=  Config->NoCollidesSpacing; 
+				rects[selected].y += Config->NoCollidesSpacing ; 
+			}
+			
+			FIT_TO_RECT(y,h,rects[selected].y,rects[selected].height);
+			if( x < rects[selected].x )
+				x = rects[selected].x ;
+		}
+	}
+	destroy_asvector( &free_space_list );
+	*px = x ; 
+	*py = y ;
+	*pmax_width = w ; 
+	*pmax_height = h ; 
+	LOCAL_DEBUG_OUT( "Final geometry %dx%d%+d%+d, Selected area = %d", w, h, x, y, selected );
+	LOCAL_DEBUG_OUT( "Current Canvas geometry %dx%d%+d%+d", WinListState.main_canvas->width, WinListState.main_canvas->height, WinListState.main_canvas->root_x, WinListState.main_canvas->root_y );
+	if( WinListState.self )
+	{
+		LOCAL_DEBUG_OUT( "Current frame geometry from AS : %lux%lu%+ld%+ld", WinListState.self->frame_rect.width, WinListState.self->frame_rect.height,WinListState.self->frame_rect.x, WinListState.self->frame_rect.y );
+		LOCAL_DEBUG_OUT( "Current frame geometry from  X : %dx%d%+d%+d", WinListState.frame_width, WinListState.frame_height,WinListState.frame_x, WinListState.frame_y );
+	}
+}
+
+void
+winlist_avoid_collision_trace( const char *caller, int line, int *px, int *py, unsigned int *pmax_width, unsigned int *pmax_height, int min_width, int min_height )
+{
+	LOCAL_DEBUG_OUT( "calling winlist_avoid_collision from %s:%d", caller, line );
+	winlist_avoid_collision( px, py, pmax_width, pmax_height, min_width, min_height );	
+}
+
+#define winlist_avoid_collision(px,py,pmax_width,pmax_height,min_width,min_height ) \
+	winlist_avoid_collision_trace( __FUNCTION__, __LINE__, px, py, pmax_width, pmax_height, min_width, min_height )	
+
+Bool
+check_avoid_collision()
+{
+	int x = WinListState.main_canvas->root_x, y = WinListState.main_canvas->root_y ; 
+    unsigned int w = (Config->MaxSize.width==0)?Scr.MyDisplayWidth:Config->MaxSize.width ;
+	unsigned int h = (Config->MaxSize.height==0)?Scr.MyDisplayHeight:Config->MaxSize.height ;
+	int min_w = 1, min_h = 1 ;
+	if( WinListState.main_canvas->height > 1 )
+	{
+		min_h = h = WinListState.main_canvas->height ; 
+		min_w = w = WinListState.main_canvas->width ; 
+	}
+    if( w > Scr.MyDisplayWidth )
+        w = Scr.MyDisplayWidth ;
+    if( h > Scr.MyDisplayHeight )
+        h = Scr.MyDisplayHeight ;
+
+	winlist_avoid_collision( &x, &y, &w, &h, min_w, min_h );
+	return ( x != WinListState.main_canvas->root_x || 
+			 y != WinListState.main_canvas->root_y ||
+			 w != WinListState.main_canvas->width ||
+			 h != WinListState.main_canvas->height ); 
+}
+
+
+
+Bool
+moveresize_main_canvas( int width, int height )
+{
+	unsigned int tmp_height = height, tmp_width = width ; 
+	int curr_x = WinListState.main_canvas->root_x;
+	int curr_y = WinListState.main_canvas->root_y; 
+	int frame_add_h = WinListState.main_canvas->bw, frame_add_v = WinListState.main_canvas->bw;
+	int new_x, new_y ; 
+	unsigned long mask = CWWidth|CWHeight  ;
+	ASFlagType changes = 0 ;
+	
+	if( WinListState.self && Config->gravity != StaticGravity ) 
+	{
+		curr_x = WinListState.frame_x ; 
+		curr_y = WinListState.frame_y ; 
+		frame_add_h = (int)WinListState.frame_width  - (int)WinListState.main_canvas->width  + (int)WinListState.frame_bw*2 ; 
+		frame_add_v = (int)WinListState.frame_height - (int)WinListState.main_canvas->height + (int)WinListState.frame_bw*2 ; 
+		if( frame_add_h < 0 )
+			frame_add_h = 0 ;
+		if( frame_add_v < 0 )
+			frame_add_v = 0 ;
+	}else
+		frame_add_h = frame_add_v = WinListState.border_width*2 ; 
+
+	tmp_width += frame_add_h ; 
+	tmp_height += frame_add_v ; 
+	new_x = curr_x ; 
+	new_y = curr_y ; 
+	
+	winlist_avoid_collision( &new_x, &new_y, &tmp_width, &tmp_height, 
+							  max(WinListState.min_col_width+frame_add_h,tmp_width), 
+							  max(WinListState.max_item_height+frame_add_v,tmp_height) );
+
+	if( new_x != curr_x )
+	{
+		mask |= CWX ; 
+		if( Config->gravity == SouthEastGravity || Config->gravity == NorthEastGravity )
+		{
+			if( get_flags(WinListState.flags, ASWL_Mapped ) && 
+			    !get_flags( Config->flags, ASWL_RowsFirst ) ) /* should not change x position in that case */
+				mask &= ~CWX ; 
+			else		
+				new_x += frame_add_h ; 
+		}else if( Config->gravity == CenterGravity || Config->gravity == StaticGravity ) 
+			new_x += frame_add_h/2 ; /* not particularly correct for StaticGravity */
+	}
+	if( new_y != curr_y ) 
+	{
+		mask |= CWY ; 
+		if( Config->gravity == SouthEastGravity || Config->gravity == SouthWestGravity )
+		{
+			if( get_flags(WinListState.flags, ASWL_Mapped ) && 
+			    get_flags( Config->flags, ASWL_RowsFirst ) ) /* should not change y position in that case */
+			{	mask &= ~CWY ;  }
+			else		
+				new_y += frame_add_v ; 
+		}else if( Config->gravity == CenterGravity || Config->gravity == StaticGravity  ) 
+			new_y += frame_add_v/2 ; 
+	}
+	changes = configure_canvas( WinListState.main_canvas, new_x, new_y, width, height, mask );
+    ASSync( False );
+	return (changes & CANVAS_RESIZED);
+}
 /* WE do redrawing in two steps :
   1) we need to determine the desired window size and resize it accordingly
   2) when we get StructureNotify event - we need to reposition and redraw
@@ -575,12 +1013,13 @@ rearrange_winlist_window( Bool dont_resize_main_canvas )
     unsigned int allowed_min_width = Config->MinSize.width ;
     unsigned int allowed_min_height = Config->MinSize.height;
     unsigned int max_col_width = (Config->MaxColWidth==0)?Scr.MyDisplayWidth:Config->MaxColWidth ;
-    unsigned int max_item_height = 0;
+    unsigned int max_item_height = 1;
+    unsigned int min_col_width = 1;
     unsigned int max_rows = 0 ;
     int row = 0, col = 0;
     unsigned int total_width = 0, total_height = 0 ;
     int y ;
-
+	
     LOCAL_DEBUG_CALLER_OUT( "%sresize canvas. windows_num = %d",
                             dont_resize_main_canvas?"Don't ":"Do ", WinListState.windows_num );
     
@@ -595,6 +1034,52 @@ rearrange_winlist_window( Bool dont_resize_main_canvas )
             allowed_max_width = Scr.MyDisplayWidth ;
         if( allowed_max_height > Scr.MyDisplayHeight )
             allowed_max_height = Scr.MyDisplayHeight ;
+	}
+	
+	if( WinListState.windows_num != 0 && WinListState.window_order != NULL )	
+	{
+    	/* Pass 1: determining size of each individual bar, as well as max height of any bar */
+    	for( i = 0 ; i < WinListState.windows_num ; ++i )
+		{
+        	ASTBarData   *tbar = WinListState.window_order[i]->bar ;
+        	int width, height ;
+        	if( tbar == NULL )
+        	{
+            	WinListState.bar_width[i] = 0 ;
+            	continue;
+        	}
+
+        	width = calculate_astbar_width( tbar );
+        	height = calculate_astbar_height( tbar );
+        	if( width == 0 )
+            	width = 1 ;
+
+        	if( width > min_col_width )
+            	min_col_width = width ;
+        	WinListState.bar_width[i] = width ;
+        	if( height > max_item_height )
+            	max_item_height = height ;
+    	}
+		LOCAL_DEBUG_OUT( "calculated max_item_height = %d", max_item_height );
+	}
+	
+	if( min_col_width <= 1 ) 
+		min_col_width = WinListState.min_col_width ; 
+	else
+		WinListState.min_col_width = min_col_width ;
+	
+	if( max_item_height <= 1 ) 
+		max_item_height = WinListState.max_item_height ; 
+	else
+		WinListState.max_item_height = max_item_height ;
+	
+	
+	if( !dont_resize_main_canvas )
+    {
+		int dumm_x = WinListState.main_canvas->root_x; 
+		int dumm_y = WinListState.main_canvas->root_y; 
+		winlist_avoid_collision( &dumm_x, &dumm_y, &allowed_max_width, &allowed_max_height, 
+								 min_col_width, max_item_height );
         if( allowed_min_width > allowed_max_width )
             allowed_min_width = allowed_max_width ;
         if( allowed_min_height > allowed_max_height )
@@ -603,16 +1088,19 @@ rearrange_winlist_window( Bool dont_resize_main_canvas )
 
     LOCAL_DEBUG_OUT( "allowed_min_size=%dx%d; allowed_max_size==%dx%d",
                      allowed_min_width, allowed_min_height, allowed_max_width, allowed_max_height );
-
+	
     if( WinListState.windows_num == 0 || WinListState.window_order == NULL )
     {
         if( !dont_resize_main_canvas )
         {
             int width = max(1,allowed_min_width);
             int height = max(1,allowed_min_height);
-            Bool invalid = ( height != WinListState.main_canvas->height || width != WinListState.main_canvas->width );
-            resize_canvas( WinListState.main_canvas, width, height );
-            if( invalid )
+			/* if( new_win_x != WinListState.main_canvas->root_x || new_win_y != WinListState.main_canvas->root_y ) 
+				moveresize_canvas( WinListState.main_canvas, new_win_x, new_win_y, width, height );
+			else
+	            resize_canvas( WinListState.main_canvas, width, height );
+			*/
+            if( moveresize_main_canvas( width, height ) )
                 return False;
         }
         if( allowed_min_width > 1 && allowed_min_height > 1 )
@@ -637,40 +1125,18 @@ rearrange_winlist_window( Bool dont_resize_main_canvas )
         return False;
     }
 
-    if( max_col_width > allowed_max_width )
-        max_col_width = allowed_max_width ;
-
-    /* Pass 1: determining size of each individual bar, as well as max height of any bar */
-    for( i = 0 ; i < WinListState.windows_num ; ++i )
-	{
-        ASTBarData   *tbar = WinListState.window_order[i]->bar ;
-        int width, height ;
-        if( tbar == NULL )
-        {
-            WinListState.bar_width[i] = 0 ;
-            continue;
-        }
-
-        width = calculate_astbar_width( tbar );
-        height = calculate_astbar_height( tbar );
-        if( width == 0 )
-            width = 1 ;
-        if( height == 0 )
-            height = 1 ;
-
-        if( width > max_col_width )
-            width = max_col_width ;
-        WinListState.bar_width[i] = width ;
-        if( height > max_item_height )
-            max_item_height = height ;
-    }
-	LOCAL_DEBUG_OUT( "calculated max_item_height = %d", max_item_height );
-	if( max_item_height <= 0 ) 
-		max_item_height = 1 ;
     max_rows = (allowed_max_height + max_item_height - 1 ) / max_item_height ;
     if( max_rows > Config->MaxRows )
         max_rows = Config->MaxRows ;
 
+   	if( max_col_width > allowed_max_width )
+       	max_col_width = allowed_max_width ;
+
+   	/* Pass 1.5: limiting sizes of the bars : */
+    for( i = 0 ; i < WinListState.windows_num ; ++i )
+        if( WinListState.bar_width[i] > max_col_width )
+            WinListState.bar_width[i] = max_col_width ;
+
     /* Pass 2: we have to decide on number of rows/columns : */
     WinListState.columns_num = WinListState.rows_num = 0 ;
     if( get_flags( Config->flags, ASWL_RowsFirst ) )
@@ -799,10 +1265,15 @@ rearrange_winlist_window( Bool dont_resize_main_canvas )
     if( !dont_resize_main_canvas )
     {
         int height = max(total_height,allowed_min_height);
-        Bool invalid = ( height != WinListState.main_canvas->height || total_width != WinListState.main_canvas->width );
-        resize_canvas( WinListState.main_canvas, total_width, height );
-        ASSync( False );
-        if( invalid )
+/*		int tmp_height = height, tmp_width = total_width ; 
+
+ 		winlist_avoid_collision( &new_win_x, &new_win_y, &tmp_width, &tmp_height );
+		if( new_win_x != WinListState.main_canvas->root_x || new_win_y != WinListState.main_canvas->root_y ) 
+			moveresize_canvas( WinListState.main_canvas, new_win_x, new_win_y, total_width, height );
+		else
+	        resize_canvas( WinListState.main_canvas, total_width, height );
+*/
+        if( moveresize_main_canvas( total_width, height ) )
             return True;
     }
 
@@ -1040,7 +1511,7 @@ configure_tbar_props( ASTBarData *tbar, ASWindowData *wd, Bool focus_only )
 		if( WinListState.tbar_props )
 		{
 			if( !get_flags( Config->set_flags, WINLIST_Align ) )
-				align = WinListState.tbar_props->align ;
+				align = DigestWinListAlign( Config, WinListState.tbar_props->align );
 			else
 				align = Config->Align ;
 		    if( !get_flags( Config->set_flags, WINLIST_HSpacing) )
@@ -1123,7 +1594,7 @@ LOCAL_DEBUG_OUT("tbar = %p, wd = %p", tbar, wd );
 	}
 }
 
-void
+Bool
 refresh_winlist_button( ASTBarData *tbar, ASWindowData *wd, Bool focus_only )
 {
 LOCAL_DEBUG_OUT("tbar = %p, wd = %p", tbar, wd );
@@ -1141,15 +1612,20 @@ LOCAL_DEBUG_OUT("tbar = %p, wd = %p", tbar, wd );
           			if( calculate_astbar_width( tbar ) > WinListState.col_width[WinListState.bar_col[i]] ||
               			calculate_astbar_height( tbar ) > WinListState.max_item_height )
 					{
-	                	rearrange_winlist_window( False );
 						rearranged = True ;
-					}
+					}else if( get_flags( wd->module_flags, ASWL_Client_NoCollides ) )
+						rearranged = check_avoid_collision();
+
+					if( rearranged ) 
+	                	rearrange_winlist_window( False );
   		        }
 				if( !rearranged )
       		    	render_winlist_button( tbar );
 			}
         }
+		return True ;
 	}
+	return False ;
 }
 
 void
@@ -1248,3 +1724,22 @@ update_winlist_styles()
 	}
 }
 
+
+void
+switch_deskviewport( int new_desk, int new_vx, int new_vy )
+{
+	Bool view_changed = (new_vx != Scr.Vx || new_vy != Scr.Vy) ;
+	Bool desk_changed = (new_desk != Scr.CurrentDesk) ;
+
+	Scr.Vx = new_vx;
+    Scr.Vy = new_vy;
+	Scr.CurrentDesk = new_desk;
+
+	if( WinListState.postpone_display  || (!view_changed && !desk_changed))
+		return ;
+	
+	if( check_avoid_collision() )
+   		rearrange_winlist_window( False );
+}
+
+
